from ..utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH


class CcrClient(NamespacedClient):
    @query_params()
    def delete_auto_follow_pattern(self, name, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-delete-auto-follow-pattern.html>`_

        :arg name: The name of the auto follow pattern.
        """
        if name in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'name'.")
        return self.transport.perform_request(
            "DELETE", _make_path("_ccr", "auto_follow", name), params=params
        )

    @query_params("wait_for_active_shards")
    def follow(self, index, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-put-follow.html>`_

        :arg index: The name of the follower index
        :arg body: The name of the leader index and other optional ccr related
            parameters
        :arg wait_for_active_shards: Sets the number of shard copies that must
            be active before returning. Defaults to 0. Set to `all` for all
            shard copies, otherwise set to any non-negative value less than or
            equal to the total number of copies for the shard (number of
            replicas + 1), default '0'
        """
        for param in (index, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "PUT", _make_path(index, "_ccr", "follow"), params=params, body=body
        )

    @query_params()
    def follow_info(self, index=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-follow-info.html>`_

        :arg index: A comma-separated list of index patterns; use `_all` to
            perform the operation on all indices
        """
        return self.transport.perform_request(
            "GET", _make_path(index, "_ccr", "info"), params=params
        )

    @query_params()
    def follow_stats(self, index, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-follow-stats.html>`_

        :arg index: A comma-separated list of index patterns; use `_all` to
            perform the operation on all indices
        """
        if index in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'index'.")
        return self.transport.perform_request(
            "GET", _make_path(index, "_ccr", "stats"), params=params
        )

    @query_params()
    def forget_follower(self, index, body, params=None):
        """
        `<http://www.elastic.co/guide/en/elasticsearch/reference/current>`_

        :arg index: the name of the leader index for which specified follower
            retention leases should be removed
        :arg body: the name and UUID of the follower index, the name of the
            cluster containing the follower index, and the alias from the
            perspective of that cluster for the remote cluster containing the
            leader index
        """
        for param in (index, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "POST",
            _make_path(index, "_ccr", "forget_follower"),
            params=params,
            body=body,
        )

    @query_params()
    def get_auto_follow_pattern(self, name=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-auto-follow-pattern.html>`_

        :arg name: The name of the auto follow pattern.
        """
        return self.transport.perform_request(
            "GET", _make_path("_ccr", "auto_follow", name), params=params
        )

    @query_params()
    def pause_follow(self, index, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-post-pause-follow.html>`_

        :arg index: The name of the follower index that should pause following
            its leader index.
        """
        if index in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'index'.")
        return self.transport.perform_request(
            "POST", _make_path(index, "_ccr", "pause_follow"), params=params
        )

    @query_params()
    def put_auto_follow_pattern(self, name, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-put-auto-follow-pattern.html>`_

        :arg name: The name of the auto follow pattern.
        :arg body: The specification of the auto follow pattern
        """
        for param in (name, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "PUT", _make_path("_ccr", "auto_follow", name), params=params, body=body
        )

    @query_params()
    def resume_follow(self, index, body=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-post-resume-follow.html>`_

        :arg index: The name of the follow index to resume following.
        :arg body: The name of the leader index and other optional ccr related
            parameters
        """
        if index in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'index'.")
        return self.transport.perform_request(
            "POST", _make_path(index, "_ccr", "resume_follow"), params=params, body=body
        )

    @query_params()
    def stats(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ccr-get-stats.html>`_
        """
        return self.transport.perform_request("GET", "/_ccr/stats", params=params)

    @query_params()
    def unfollow(self, index, params=None):
        """
        `<http://www.elastic.co/guide/en/elasticsearch/reference/current>`_

        :arg index: The name of the follower index that should be turned into a
            regular index.
        """
        if index in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'index'.")
        return self.transport.perform_request(
            "POST", _make_path(index, "_ccr", "unfollow"), params=params
        )



from ..utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH


class SecurityClient(NamespacedClient):
    @query_params()
    def authenticate(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-authenticate.html>`_
        """
        return self.transport.perform_request(
            "GET", "/_security/_authenticate", params=params
        )

    @query_params("refresh")
    def change_password(self, body, username=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-change-password.html>`_

        :arg body: the new password for the user
        :arg username: The username of the user to change the password for
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "PUT",
            _make_path("_security", "user", username, "_password"),
            params=params,
            body=body,
        )

    @query_params("usernames")
    def clear_cached_realms(self, realms, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-clear-cache.html>`_

        :arg realms: Comma-separated list of realms to clear
        :arg usernames: Comma-separated list of usernames to clear from the
            cache
        """
        if realms in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'realms'.")
        return self.transport.perform_request(
            "POST",
            _make_path("_security", "realm", realms, "_clear_cache"),
            params=params,
        )

    @query_params()
    def clear_cached_roles(self, name, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-clear-role-cache.html>`_

        :arg name: Role name
        """
        if name in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'name'.")
        return self.transport.perform_request(
            "POST", _make_path("_security", "role", name, "_clear_cache"), params=params
        )

    @query_params("refresh")
    def create_api_key(self, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-create-api-key.html>`_

        :arg body: The api key request to create an API key
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "PUT", "/_security/api_key", params=params, body=body
        )

    @query_params("refresh")
    def delete_privileges(self, application, name, params=None):
        """
        `<TODO>`_

        :arg application: Application name
        :arg name: Privilege name
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        for param in (application, name):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "DELETE",
            _make_path("_security", "privilege", application, name),
            params=params,
        )

    @query_params("refresh")
    def delete_role(self, name, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-delete-role.html>`_

        :arg name: Role name
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if name in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'name'.")
        return self.transport.perform_request(
            "DELETE", _make_path("_security", "role", name), params=params
        )

    @query_params("refresh")
    def delete_role_mapping(self, name, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-delete-role-mapping.html>`_

        :arg name: Role-mapping name
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if name in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'name'.")
        return self.transport.perform_request(
            "DELETE", _make_path("_security", "role_mapping", name), params=params
        )

    @query_params("refresh")
    def delete_user(self, username, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-delete-user.html>`_

        :arg username: username
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if username in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'username'.")
        return self.transport.perform_request(
            "DELETE", _make_path("_security", "user", username), params=params
        )

    @query_params("refresh")
    def disable_user(self, username, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-disable-user.html>`_

        :arg username: The username of the user to disable
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if username in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'username'.")
        return self.transport.perform_request(
            "PUT", _make_path("_security", "user", username, "_disable"), params=params
        )

    @query_params("refresh")
    def enable_user(self, username, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-enable-user.html>`_

        :arg username: The username of the user to enable
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if username in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'username'.")
        return self.transport.perform_request(
            "PUT", _make_path("_security", "user", username, "_enable"), params=params
        )

    @query_params("id", "name", "realm_name", "username")
    def get_api_key(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-get-api-key.html>`_

        :arg id: API key id of the API key to be retrieved
        :arg name: API key name of the API key to be retrieved
        :arg realm_name: realm name of the user who created this API key to be
            retrieved
        :arg username: user name of the user who created this API key to be
            retrieved
        """
        return self.transport.perform_request(
            "GET", "/_security/api_key", params=params
        )

    @query_params()
    def get_privileges(self, application=None, name=None, params=None):
        """
        `<TODO>`_

        :arg application: Application name
        :arg name: Privilege name
        """
        return self.transport.perform_request(
            "GET",
            _make_path("_security", "privilege", application, name),
            params=params,
        )

    @query_params()
    def get_role(self, name=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-get-role.html>`_

        :arg name: Role name
        """
        return self.transport.perform_request(
            "GET", _make_path("_security", "role", name), params=params
        )

    @query_params()
    def get_role_mapping(self, name=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-get-role-mapping.html>`_

        :arg name: Role-Mapping name
        """
        return self.transport.perform_request(
            "GET", _make_path("_security", "role_mapping", name), params=params
        )

    @query_params()
    def get_token(self, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-get-token.html>`_

        :arg body: The token request to get
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "POST", "/_security/oauth2/token", params=params, body=body
        )

    @query_params()
    def get_user(self, username=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-get-user.html>`_

        :arg username: A comma-separated list of usernames
        """
        return self.transport.perform_request(
            "GET", _make_path("_security", "user", username), params=params
        )

    @query_params()
    def get_user_privileges(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-get-user-privileges.html>`_
        """
        return self.transport.perform_request(
            "GET", "/_security/user/_privileges", params=params
        )

    @query_params()
    def has_privileges(self, body, user=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-has-privileges.html>`_

        :arg body: The privileges to test
        :arg user: Username
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "GET",
            _make_path("_security", "user", user, "_has_privileges"),
            params=params,
            body=body,
        )

    @query_params()
    def invalidate_api_key(self, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-invalidate-api-key.html>`_

        :arg body: The api key request to invalidate API key(s)
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "DELETE", "/_security/api_key", params=params, body=body
        )

    @query_params()
    def invalidate_token(self, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-invalidate-token.html>`_

        :arg body: The token to invalidate
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "DELETE", "/_security/oauth2/token", params=params, body=body
        )

    @query_params("refresh")
    def put_privileges(self, body, params=None):
        """
        `<TODO>`_

        :arg body: The privilege(s) to add
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "PUT", "/_security/privilege/", params=params, body=body
        )

    @query_params("refresh")
    def put_role(self, name, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-put-role.html>`_

        :arg name: Role name
        :arg body: The role to add
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        for param in (name, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "PUT", _make_path("_security", "role", name), params=params, body=body
        )

    @query_params("refresh")
    def put_role_mapping(self, name, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-put-role-mapping.html>`_

        :arg name: Role-mapping name
        :arg body: The role to add
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        for param in (name, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "PUT",
            _make_path("_security", "role_mapping", name),
            params=params,
            body=body,
        )

    @query_params("refresh")
    def put_user(self, username, body, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-put-user.html>`_

        :arg username: The username of the User
        :arg body: The user to add
        :arg refresh: If `true` (the default) then refresh the affected shards
            to make this operation visible to search, if `wait_for` then wait
            for a refresh to make this operation visible to search, if `false`
            then do nothing with refreshes., valid choices are: 'true', 'false',
            'wait_for'
        """
        for param in (username, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "PUT", _make_path("_security", "user", username), params=params, body=body
        )



from ..utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH


class LicenseClient(NamespacedClient):
    @query_params()
    def delete(self, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_
        """
        return self.transport.perform_request("DELETE", "/_license", params=params)

    @query_params("local")
    def get(self, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_

        :arg local: Return local information, do not retrieve the state from
            master node (default: false)
        """
        return self.transport.perform_request("GET", "/_license", params=params)

    @query_params()
    def get_basic_status(self, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_
        """
        return self.transport.perform_request(
            "GET", "/_license/basic_status", params=params
        )

    @query_params()
    def get_trial_status(self, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_
        """
        return self.transport.perform_request(
            "GET", "/_license/trial_status", params=params
        )

    @query_params("acknowledge")
    def post(self, body=None, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_

        :arg body: licenses to be installed
        :arg acknowledge: whether the user has acknowledged acknowledge messages
            (default: false)
        """
        return self.transport.perform_request(
            "PUT", "/_license", params=params, body=body
        )

    @query_params("acknowledge")
    def post_start_basic(self, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_

        :arg acknowledge: whether the user has acknowledged acknowledge messages
            (default: false)
        """
        return self.transport.perform_request(
            "POST", "/_license/start_basic", params=params
        )

    @query_params("acknowledge", "doc_type")
    def post_start_trial(self, params=None):
        """
        `<https://www.elastic.co/guide/en/x-pack/current/license-management.html>`_

        :arg acknowledge: whether the user has acknowledged acknowledge messages
            (default: false)
        :arg doc_type: The type of trial license to generate (default: "trial")
        """
        return self.transport.perform_request(
            "POST", "/_license/start_trial", params=params
        )



from ..utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH


class IlmClient(NamespacedClient):
    @query_params()
    def delete_lifecycle(self, policy=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-delete-lifecycle.html>`_

        :arg policy: The name of the index lifecycle policy
        """
        return self.transport.perform_request(
            "DELETE", _make_path("_ilm", "policy", policy), params=params
        )

    @query_params()
    def explain_lifecycle(self, index=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-explain-lifecycle.html>`_

        :arg index: The name of the index to explain
        """
        return self.transport.perform_request(
            "GET", _make_path(index, "_ilm", "explain"), params=params
        )

    @query_params()
    def get_lifecycle(self, policy=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-get-lifecycle.html>`_

        :arg policy: The name of the index lifecycle policy
        """
        return self.transport.perform_request(
            "GET", _make_path("_ilm", "policy", policy), params=params
        )

    @query_params()
    def get_status(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-get-status.html>`_
        """
        return self.transport.perform_request("GET", "/_ilm/status", params=params)

    @query_params()
    def move_to_step(self, index=None, body=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-move-to-step.html>`_

        :arg index: The name of the index whose lifecycle step is to change
        :arg body: The new lifecycle step to move to
        """
        return self.transport.perform_request(
            "POST", _make_path("_ilm", "move", index), params=params, body=body
        )

    @query_params()
    def put_lifecycle(self, policy=None, body=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-put-lifecycle.html>`_

        :arg policy: The name of the index lifecycle policy
        :arg body: The lifecycle policy definition to register
        """
        return self.transport.perform_request(
            "PUT", _make_path("_ilm", "policy", policy), params=params, body=body
        )

    @query_params()
    def remove_policy(self, index=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-remove-policy.html>`_

        :arg index: The name of the index to remove policy on
        """
        return self.transport.perform_request(
            "POST", _make_path(index, "_ilm", "remove"), params=params
        )

    @query_params()
    def retry(self, index=None, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-retry-policy.html>`_

        :arg index: The name of the indices (comma-separated) whose failed
            lifecycle step is to be retry
        """
        return self.transport.perform_request(
            "POST", _make_path(index, "_ilm", "retry"), params=params
        )

    @query_params()
    def start(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-start.html>`_
        """
        return self.transport.perform_request("POST", "/_ilm/start", params=params)

    @query_params()
    def stop(self, params=None):
        """
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/ilm-stop.html>`_
        """
        return self.transport.perform_request("POST", "/_ilm/stop", params=params)



from ..utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH


class SqlClient(NamespacedClient):
    @query_params()
    def clear_cursor(self, body, params=None):
        """
        `<Clear SQL cursor>`_

        :arg body: Specify the cursor value in the `cursor` element to clean the
            cursor.
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "POST", "/_sql/close", params=params, body=body
        )

    @query_params("format")
    def query(self, body, params=None):
        """
        `<Execute SQL>`_

        :arg body: Use the `query` element to start a query. Use the `cursor`
            element to continue a query.
        :arg format: a short version of the Accept header, e.g. json, yaml
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request("POST", "/_sql", params=params, body=body)

    @query_params()
    def translate(self, body, params=None):
        """
        `<Translate SQL into Elasticsearch queries>`_

        :arg body: Specify the query in the `query` element.
        """
        if body in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'body'.")
        return self.transport.perform_request(
            "POST", "/_sql/translate", params=params, body=body
        )



from .errors import BulkIndexError, ScanError
from .actions import expand_action, streaming_bulk, bulk, parallel_bulk
from .actions import scan, reindex
from .actions import _chunk_actions, _process_bulk_chunk



from ..utils import NamespacedClient, query_params

from .ccr import CcrClient
from .data_frame import Data_FrameClient
from .deprecation import DeprecationClient
from .graph import GraphClient
from .ilm import IlmClient
from .indices import IndicesClient
from .license import LicenseClient
from .migration import MigrationClient
from .ml import MlClient
from .monitoring import MonitoringClient
from .rollup import RollupClient
from .security import SecurityClient
from .sql import SqlClient
from .ssl import SslClient
from .watcher import WatcherClient


class XPackClient(NamespacedClient):
    namespace = "xpack"

    def __init__(self, *args, **kwargs):
        super(XPackClient, self).__init__(*args, **kwargs)
        self.ccr = CcrClient(self.client)
        self.data_frame = Data_FrameClient(self.client)
        self.deprecation = DeprecationClient(self.client)
        self.graph = GraphClient(self.client)
        self.ilm = IlmClient(self.client)
        self.indices = IndicesClient(self.client)
        self.license = LicenseClient(self.client)
        self.migration = MigrationClient(self.client)
        self.ml = MlClient(self.client)
        self.monitoring = MonitoringClient(self.client)
        self.rollup = RollupClient(self.client)
        self.security = SecurityClient(self.client)
        self.sql = SqlClient(self.client)
        self.ssl = SslClient(self.client)
        self.watcher = WatcherClient(self.client)

    @query_params("categories", "human")
    def info(self, params=None):
        """
        Retrieve information about xpack, including build number/timestamp and license status
        `<https://www.elastic.co/guide/en/elasticsearch/reference/current/info-api.html>`_

        :arg categories: Comma-separated list of info categories. Can be any of:
            build, license, features
        :arg human: Presents additional info for humans (feature descriptions
            and X-Pack tagline)
        """
        return self.transport.perform_request("GET", "/_xpack", params=params)

    @query_params("master_timeout")
    def usage(self, params=None):
        """
        Retrieve information about xpack features usage

        :arg master_timeout: Specify timeout for watch write operation
        """
        return self.transport.perform_request("GET", "/_xpack/usage", params=params)



from ..utils import NamespacedClient, query_params, _make_path, SKIP_IN_PATH


class RollupClient(NamespacedClient):
    @query_params()
    def delete_job(self, id, params=None):
        """
        `<>`_

        :arg id: The ID of the job to delete
        """
        if id in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'id'.")
        return self.transport.perform_request(
            "DELETE", _make_path("_rollup", "job", id), params=params
        )

    @query_params()
    def get_jobs(self, id=None, params=None):
        """
        `<>`_

        :arg id: The ID of the job(s) to fetch. Accepts glob patterns, or left
            blank for all jobs
        """
        return self.transport.perform_request(
            "GET", _make_path("_rollup", "job", id), params=params
        )

    @query_params()
    def get_rollup_caps(self, id=None, params=None):
        """
        `<>`_

        :arg id: The ID of the index to check rollup capabilities on, or left
            blank for all jobs
        """
        return self.transport.perform_request(
            "GET", _make_path("_rollup", "data", id), params=params
        )

    @query_params()
    def get_rollup_index_caps(self, index, params=None):
        """
        `<>`_

        :arg index: The rollup index or index pattern to obtain rollup
            capabilities from.
        """
        if index in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'index'.")
        return self.transport.perform_request(
            "GET", _make_path(index, "_rollup", "data"), params=params
        )

    @query_params()
    def put_job(self, id, body, params=None):
        """
        `<>`_

        :arg id: The ID of the job to create
        :arg body: The job configuration
        """
        for param in (id, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "PUT", _make_path("_rollup", "job", id), params=params, body=body
        )

    @query_params("rest_total_hits_as_int", "typed_keys")
    def rollup_search(self, index, body, doc_type=None, params=None):
        """
        `<>`_

        :arg index: The indices or index-pattern(s) (containing rollup or
            regular data) that should be searched
        :arg body: The search request body
        :arg doc_type: The doc type inside the index
        :arg rest_total_hits_as_int: Indicates whether hits.total should be
            rendered as an integer or an object in the rest search response,
            default False
        :arg typed_keys: Specify whether aggregation and suggester names should
            be prefixed by their respective types in the response
        """
        for param in (index, body):
            if param in SKIP_IN_PATH:
                raise ValueError("Empty value passed for a required argument.")
        return self.transport.perform_request(
            "GET",
            _make_path(index, doc_type, "_rollup_search"),
            params=params,
            body=body,
        )

    @query_params()
    def start_job(self, id, params=None):
        """
        `<>`_

        :arg id: The ID of the job to start
        """
        if id in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'id'.")
        return self.transport.perform_request(
            "POST", _make_path("_rollup", "job", id, "_start"), params=params
        )

    @query_params("timeout", "wait_for_completion")
    def stop_job(self, id, params=None):
        """
        `<>`_

        :arg id: The ID of the job to stop
        :arg timeout: Block for (at maximum) the specified duration while
            waiting for the job to stop.  Defaults to 30s.
        :arg wait_for_completion: True if the API should block until the job has
            fully stopped, false if should be executed async. Defaults to false.
        """
        if id in SKIP_IN_PATH:
            raise ValueError("Empty value passed for a required argument 'id'.")
        return self.transport.perform_request(
            "POST", _make_path("_rollup", "job", id, "_stop"), params=params
        )



from ..utils import NamespacedClient, query_params, _make_path


class DeprecationClient(NamespacedClient):
    @query_params()
    def info(self, index=None, params=None):
        """
        `<http://www.elastic.co/guide/en/migration/current/migration-api-deprecation.html>`_

        :arg index: Index pattern
        """
        return self.transport.perform_request(
            "GET",
            _make_path(index, "_xpack", "migration", "deprecations"),
            params=params,
        )
